from typing import List, Tuple

from sqlalchemy import create_engine
from enums.DatabaseDialectEnum import DatabaseDialectEnum
from components.Model.InsumoModel import InsumoModel
from components.Databases.Enums.MysqlDataTypeEnum import MysqlDataTypeEnum
from components.Databases.Enums.MysqlDataTypeORMEnum import MysqlDataTypeORMEnum


from sqlalchemy import Table, Column, MetaData
import logging

logger = logging.getLogger()


class Mysql:

    def __init__(self, host: str, port: int, user: str, password: str, database: str) -> None:
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.database = database
        self.engine = None

    def create_engine(self) -> None:
        try:
            dialect = DatabaseDialectEnum.MYSQL.value
            url = f"{dialect}://{self.user}:{self.password}@{self.host}:{str(self.port)}/{self.database}?charset=utf8mb4"
            self.engine = create_engine(url)
        except Exception as e:
            logger.error(f"Error creando engine de Mysql. {e}")

    def create_model(self, tablename: str, fields: List[Tuple[str]], modelName: str = "TableModel") -> bool:
        model = None
        try:
            model = type(modelName, (InsumoModel,), {
                '__tablename__': tablename,
                '__table_args__': {'extend_existing': True}
            })
            for field in fields:
                logger.debug(f"Attribute: {field}")
                name = field[0]
                if field[2] != -1:
                    size = int(field[2] / 4)
                    try:
                        if not isinstance(field[3], type(None)) and field[3] > 0:
                            data_type = MysqlDataTypeORMEnum[MysqlDataTypeEnum(field[1]).name].value(precision=field[2], scale=field[3])
                        else:
                            data_type = MysqlDataTypeORMEnum[MysqlDataTypeEnum(field[1]).name].value(size)
                    except TypeError:
                        data_type = MysqlDataTypeORMEnum[MysqlDataTypeEnum(field[1]).name].value()
                else:
                    data_type = MysqlDataTypeORMEnum[MysqlDataTypeEnum(field[1]).name].value()
                setattr(model, name, Column(data_type))
            model = model.__table__
        except Exception as e:
            logger.error(f"Error creando modelo dinámico en Mysql con nombre {tablename}. {e}")
        finally:
            return model

    def generate_sql_procedure(self, command: str, reserved_word: str = "execute") -> str:
        response = ""
        try:
            command = command.replace(reserved_word, "").replace(";", "")
            response = f"call {command};"
            logger.debug("COMANDO MYSQL:", response)
        except Exception as e:
            logger.error(f"Error generando comando sql para procedure Mysql. Comando: {command}. {e}")
        finally:
            return response
